@@ -12,10 +12,11 @@ import logging
 from concurrent.futures import ProcessPoolExecutor
 from pathlib import Path
 
+import numpy as np
+
 import gprof_nn.logging
 from gprof_nn.retrieval import RetrievalDriver, RetrievalGradientDriver
 from gprof_nn import sensors
-from gprof_nn import statistics
 
 
 LOGGER = logging.getLogger(__name__)
@@ -62,33 +63,65 @@ def add_parser(subparsers):
     parser.add_argument("--no_resampling",
                         help="Disable latitude resampling of sensor.",
                         action="store_true")
+    parser.add_argument("--latitude_ratios",
+                        help=("A latitude-time probability ratio file to use"
+                              " for resampling the training data"),
+                        type=str,
+                        default=None)
     parser.set_defaults(func=run)
 
 
-STATS = {
-    "training_1d": [
-        statistics.TrainingDataStatistics(kind="1d"),
-        statistics.ZonalDistribution(),
-        statistics.GlobalDistribution()
-    ],
-    "training_3d": [
-        statistics.TrainingDataStatistics(kind="3d"),
-        statistics.ZonalDistribution(),
-        statistics.GlobalDistribution()
-    ],
-    "bin": [
-        statistics.BinFileStatistics(),
-        statistics.GlobalDistribution()
-    ],
-    "observations": [statistics.ObservationStatistics()],
-    "retrieval": [
-        statistics.RetrievalStatistics(),
-        statistics.ZonalDistribution(),
-        statistics.GlobalDistribution(),
-        statistics.ScanPositionMean()
-        ],
-    "combined": [statistics.GPMCMBStatistics(monthly=False)]
-}
+def get_stats(kind, latitude_ratios):
+    """
+    Provide list of statistic types for given statistics type.
+
+    Args:
+        kind: The kind of statistics to be computed
+        latitude_ratios: Optional array with latitude or latitude-time
+            ratios to use to resample the observations.
+
+    Return:
+        A list with statistics object to calculate the requested
+        statistics type.
+    """
+    from gprof_nn import statistics
+    if kind == "training_1d":
+        stats = [
+            statistics.TrainingDataStatistics(kind="1d"),
+            statistics.ZonalDistribution(),
+            statistics.GlobalDistribution()
+        ]
+    elif kind == "training_3d":
+        stats = [
+            statistics.TrainingDataStatistics(kind="3d"),
+            statistics.ZonalDistribution(),
+            statistics.GlobalDistribution()
+        ]
+    elif kind == "bin":
+        stats = [
+            statistics.BinFileStatistics(),
+            statistics.GlobalDistribution()
+        ]
+    elif kind == "observations":
+        stats = [statistics.ObservationStatistics(statistics=latitude_ratios)]
+    elif kind == "retrieval":
+        stats = [
+            statistics.RetrievalStatistics(statistics=latitude_ratios),
+            statistics.ZonalDistribution(statistics=latitude_ratios),
+            statistics.GlobalDistribution(),
+            statistics.ScanPositionMean()
+        ]
+    elif kind == "combined":
+        stats = [statistics.GPMCMBStatistics(monthly=False)]
+    else:
+        LOGGER.error(
+            "Kind must be one of ['training_1d', 'training_3d', "
+            "'bin', 'observations', 'retrieval' 'combined'] not"
+            "%s", kind
+        )
+        return None
+    return stats
+
 
 PATTERNS = {
     "training_1d": "**/*.nc*",
@@ -107,6 +140,7 @@ def run(args):
     Args:
         args: The namespace object provided by the top-level parser.
     """
+    from gprof_nn import statistics
 
     #
     # Check and load inputs.
@@ -124,16 +158,15 @@ def run(args):
     if args.no_correction:
         sensor._correction = None
 
+    latitude_ratios = None
+    if args.latitude_ratios is not None:
+        sensor._latitude_ratios = args.latitude_ratios
+        latitude_ratios = np.load(args.latitude_ratios)
+
     kind = args.kind.lower().strip()
     if kind == "training":
         kind = "training_1d"
 
-    if not kind in STATS.keys():
-        LOGGER.error(
-            f"'kind' argument must be one of {list(STATS.keys())}."
-        )
-        return 1
-
     inputs = [Path(f) for f in args.input]
     for path in inputs:
         if not path.exists():
@@ -157,6 +190,9 @@ def run(args):
     for path in inputs:
         input_files += list(path.glob(endings))
 
-    stats = STATS[kind]
+    stats = get_stats(kind, latitude_ratios)
+    if stats is None:
+        return 1
+
     processor = statistics.StatisticsProcessor(sensor, input_files, stats)
     processor.run(n_procs, output)
