@@ -56,14 +56,21 @@ class L1CFile:
             month = date.month
             day = date.day
             path = Path(path) / f"{year:02}{month:02}" / f"{year:02}{month:02}{day:02}"
-            files = path.glob(sensor.l1c_file_prefix + f"*{granule:06}.V05A.HDF5")
+            files = path.glob(sensor.l1c_file_prefix + f"*.{granule:06}.*.HDF5")
         else:
             path = Path(path)
             files = path.glob(
-                "**/" + sensor.l1c_file_prefix + f"*{granule:06}.V05A.HDF5"
+                "**/" + sensor.l1c_file_prefix + f"*.{granule:06}.*.HDF5"
             )
 
+        files = list(files)
+        if len(files) > 1:
+            raise Exception(
+                f"Found more than 1 matching L1C file. This indicates something"
+                f"went wrong. Found the following files: {files}"
+            )
         try:
+            f = files[0]
             f = next(iter(files))
             return L1CFile(f)
         except StopIteration:
@@ -92,7 +99,7 @@ class L1CFile:
         month = date.month
         day = date.day
         data_path = Path(path) / f"{year:02}{month:02}" / f"{year:02}{month:02}{day:02}"
-        files = list(data_path.glob(sensor.l1c_file_prefix + "*.V05A.HDF5"))
+        files = list(data_path.glob(sensor.l1c_file_prefix + "*.HDF5"))
 
         # Add files from following day.
         date_f = date + pd.DateOffset(1)
@@ -100,7 +107,7 @@ class L1CFile:
         month = date_f.month
         day = date_f.day
         data_path = Path(path) / f"{year:02}{month:02}" / f"{year:02}{month:02}{day:02}"
-        files += list(data_path.glob(sensor.l1c_file_prefix + "*.V05A.HDF5"))
+        files += list(data_path.glob(sensor.l1c_file_prefix + "*.HDF5"))
 
         # Add files from previous day.
         date_f = date - pd.DateOffset(1)
@@ -109,9 +116,9 @@ class L1CFile:
         day = date_f.day
         data_path = Path(path) / f"{year:02}{month:02}" / f"{year:02}{month:02}{day:02}"
 
-        files += list(data_path.glob(sensor.l1c_file_prefix + "*.V05A.HDF5"))
+        files += list(data_path.glob(sensor.l1c_file_prefix + "*.HDF5"))
 
-        files += list(path.glob(sensor.l1c_file_prefix + "*.V05A.HDF5"))
+        files += list(path.glob(sensor.l1c_file_prefix + "*.HDF5"))
 
         start_times = []
         end_times = []
@@ -165,16 +172,19 @@ class L1CFile:
         data_path = Path(path) / f"{year:02}{month:02}" / f"{year:02}{month:02}{day:02}"
         files = list(
             data_path.glob(
-                sensor.l1c_file_prefix + f"*{date.year:04}{month:02}{day:02}*.V05A.HDF5"
+                sensor.l1c_file_prefix + f"*{date.year:04}{month:02}{day:02}*.HDF5"
             )
         )
         files += list(
             path.glob(
-                sensor.l1c_file_prefix + f"*{date.year:04}{month:02}{day:02}*.V05A.HDF5"
+                sensor.l1c_file_prefix + f"*{date.year:04}{month:02}{day:02}*.HDF5"
             )
         )
         for f in files:
-            f = L1CFile(f)
+            try:
+                f = L1CFile(f)
+            except Exception:
+                continue
             if roi is not None:
                 if f.covers_roi(roi):
                     yield f
@@ -193,9 +203,11 @@ class L1CFile:
 
         import h5py
         with h5py.File(self.path, "r") as data:
-            header = data.attrs["FileHeader"].decode().split()[6:8]
-            satellite = header[0].split("=")[1][:-1]
-            sensor = header[1].split("=")[1][:-1]
+            header = data.attrs["FileHeader"].decode().splitlines()
+            satellite = header[6].split("=")[1][:-1]
+            sensor = header[7].split("=")[1][:-1]
+            self.header = header
+            self.granule = int(header[11].split("=")[1][:-1])
             date = self.start_time
             self.sensor = sensors.get_sensor(
                 sensor,
@@ -236,7 +248,7 @@ class L1CFile:
         """String representation for file."""
         return f"L1CFile(filename='{self.path.name}')"
 
-    def extract_scans(self, roi, output_filename):
+    def extract_scans(self, roi, output_filename, min_scans=None):
         """
         Extract scans over a rectangular region of interest (ROI).
 
@@ -257,121 +269,80 @@ class L1CFile:
 
             indices = np.where(
                 np.any(
-                    (lats > lat_min)
+                    (lats >= lat_min)
                     * (lats < lat_max)
-                    * (lons > lon_min)
+                    * (lons >= lon_min)
                     * (lons < lon_max),
                     axis=-1,
                 )
             )[0]
             d_inds = np.diff(indices)
             if any(d_inds > 1):
-                ind = np.where(d_inds > 1)[0]
+                ind = np.where(d_inds > 1)[0][0]
                 indices = indices[:ind]
 
+            if len(indices) > 0:
+                i_start, i_end = indices[[0, -1]]
+                n_scans = i_end - i_start
+                if min_scans is not None and n_scans < min_scans:
+                    diff_l = (min_scans - n_scans) // 2
+                    i_start = max(0, i_start - diff_l)
+                    diff_r = min_scans - (i_end - i_start)
+                    i_end = i_end + diff_r
+            else:
+                i_start = 0
+                i_end = 0
+            scans = slice(i_start, i_end)
+
             with h5py.File(output_filename, "w") as output:
 
-                g = output.create_group("S1")
-                n_scans = indices.size
-                for name, item in input["S1"].items():
-                    if isinstance(item, h5py.Dataset):
-                        shape = item.shape
-                        g.create_dataset(
-                            name, shape=(n_scans,) + shape[1:], data=item[indices]
-                        )
+                group_index = 1
+                while f"S{group_index}" in input:
 
-                for a in input["S1"].attrs:
-                    s = input["S1"].attrs[a].decode()
-                    s = _RE_META_INFO.sub(f"NumberScansGranule={n_scans};", s)
-                    s = np.bytes_(s)
-                    g.attrs[a] = s
+                    group_name = f"S{group_index}"
 
-                g_st = g.create_group("ScanTime")
-                for name, item in input["S1/ScanTime"].items():
-                    if isinstance(item, h5py.Dataset):
-                        shape = item.shape
-                        g_st.create_dataset(
-                            name, shape=(n_scans,) + shape[1:], data=item[indices]
-                        )
-
-                g_sc = g.create_group("SCstatus")
-                for name, item in input["S1/SCstatus"].items():
-                    if isinstance(item, h5py.Dataset):
-                        shape = item.shape
-                        g_sc.create_dataset(
-                            name, shape=(n_scans,) + shape[1:], data=item[indices]
-                        )
-
-                if "S2" in input.keys():
-                    g = output.create_group("S2")
-                    for name, item in input["S2"].items():
+                    g = output.create_group(group_name)
+                    n_scans = i_end - i_start
+                    for name, item in input[group_name].items():
                         if isinstance(item, h5py.Dataset):
                             shape = item.shape
                             g.create_dataset(
-                                name, shape=(n_scans,) + shape[1:], data=item[indices]
+                                name, shape=(n_scans,) + shape[1:], data=item[scans]
                             )
-                    for a in input["S2"].attrs:
-                        s = input["S2"].attrs[a].decode()
+
+                    for a in input[group_name].attrs:
+                        s = input[group_name].attrs[a].decode()
                         s = _RE_META_INFO.sub(f"NumberScansGranule={n_scans};", s)
                         s = np.bytes_(s)
                         g.attrs[a] = s
 
                     g_st = g.create_group("ScanTime")
-                    for name, item in input["S2/ScanTime"].items():
+                    for name, item in input[f"{group_name}/ScanTime"].items():
                         if isinstance(item, h5py.Dataset):
                             shape = item.shape
                             g_st.create_dataset(
-                                name, shape=(n_scans,) + shape[1:], data=item[indices]
+                                name, shape=(n_scans,) + shape[1:], data=item[scans]
                             )
 
                     g_sc = g.create_group("SCstatus")
-                    for name, item in input["S2/SCstatus"].items():
+                    for name, item in input[f"{group_name}/SCstatus"].items():
                         if isinstance(item, h5py.Dataset):
                             shape = item.shape
                             g_sc.create_dataset(
-                                name, shape=(n_scans,) + shape[1:], data=item[indices]
+                                name, shape=(n_scans,) + shape[1:], data=item[scans]
                             )
 
-                if "S3" in input.keys():
-                    g = output.create_group("S3")
-                    for name, item in input["S3"].items():
-                        if isinstance(item, h5py.Dataset):
-                            shape = item.shape
-                            g.create_dataset(
-                                name, shape=(n_scans,) + shape[1:], data=item[indices]
-                            )
-                    for a in input["S3"].attrs:
-                        s = input["S3"].attrs[a].decode()
-                        s = _RE_META_INFO.sub(f"NumberScansGranule={n_scans};", s)
-                        s = np.bytes_(s)
-                        g.attrs[a] = s
+                    group_index += 1
 
-                    g_st = g.create_group("ScanTime")
-                    for name, item in input["S3/ScanTime"].items():
-                        if isinstance(item, h5py.Dataset):
-                            shape = item.shape
-                            g_st.create_dataset(
-                                name, shape=(n_scans,) + shape[1:], data=item[indices]
-                            )
-
-                    g_sc = g.create_group("SCstatus")
-                    for name, item in input["S3/SCstatus"].items():
-                        if isinstance(item, h5py.Dataset):
-                            shape = item.shape
-                            g_sc.create_dataset(
-                                name, shape=(n_scans,) + shape[1:], data=item[indices]
-                            )
                 for a in input.attrs:
                     output.attrs[a] = input.attrs[a]
 
-        return indices.min(), indices.max()
+        return i_start, i_end
 
-    def extract_scans_and_pixels(self, scans, output_filename, n_pixels=-1):
+    def extract_scan_range(self, scan_start, scan_end, output_filename):
         """
-        Extract first pixel from each scan in file.
-
-        The main purposed of this method is to simplify the generation
-        of small files for testing purposes.
+        Extract a range of scans from the L1C file and write to new
+        L1C file.
 
         Args:
             scans: Indices of the scans to extract.
@@ -380,95 +351,62 @@ class L1CFile:
         """
         import h5py
         with h5py.File(self.path, "r") as input:
-            if n_pixels < 0:
-                n_pixels = input["S1/Latitude"].shape[1]
-
-            lats = input["S1/Latitude"][scans, :n_pixels]
-            lons = input["S1/Longitude"][scans, :n_pixels]
 
             with h5py.File(output_filename, "w") as output:
 
-                g = output.create_group("S1")
-                n_scans = len(scans)
-                n_pixels = 1
-                for name, item in input["S1"].items():
-                    if isinstance(item, h5py.Dataset):
-                        shape = item.shape
-                        g.create_dataset(
-                            name,
-                            shape=(n_scans, n_pixels) + shape[2:],
-                            data=item[scans, :n_pixels],
-                        )
+                n_scans_total = input["S1/Latitude"].shape[0]
 
-                g_st = g.create_group("ScanTime")
-                for name, item in input["S1/ScanTime"].items():
-                    if isinstance(item, h5py.Dataset):
-                        shape = item.shape
-                        if len(shape) > 1:
+                i = 1
+                while f"S{i}" in input:
+                    group_name = f"S{i}"
+
+                    shape = input[f"{group_name}/Latitude"].shape
+                    n_scans = shape[0]
+                    scaling = n_scans // n_scans_total
+                    n_scans = (scan_end - scan_start) * scaling
+                    scan_range = slice(
+                        scan_start * scaling,
+                        scan_end * scaling
+                    )
+                    print(scaling, scan_range)
+
+                    g = output.create_group(group_name)
+                    for name, item in input[group_name].items():
+                        if isinstance(item, h5py.Dataset):
+                            print(name, item.shape,)
+                            shape = item.shape
+                            g.create_dataset(
+                                name,
+                                shape=(n_scans,) + shape[1:],
+                                data=item[scan_range],
+                            )
+
+                    for a in input[group_name].attrs:
+                        s = input[group_name].attrs[a].decode()
+                        s = _RE_META_INFO.sub(f"NumberScansGranule={n_scans};", s)
+                        s = np.bytes_(s)
+                        g.attrs[a] = s
+
+                    g_st = g.create_group("ScanTime")
+                    for name, item in input[f"{group_name}/ScanTime"].items():
+                        if isinstance(item, h5py.Dataset):
+                            shape = item.shape
                             g_st.create_dataset(
                                 name,
-                                shape=(n_scans, n_pixels) + shape[2:],
-                                data=item[scans, :n_pixels],
-                            )
-                        else:
-                            g_st.create_dataset(
-                                name, shape=(n_scans,), data=item[scans]
+                                shape=(n_scans,) + shape[1:],
+                                data=item[scan_range],
                             )
 
-                g_sc = g.create_group("SCstatus")
-                for name, item in input["S1/SCstatus"].items():
-                    if isinstance(item, h5py.Dataset):
-                        shape = item.shape
-                        if len(shape) > 1:
+                    g_sc = g.create_group("SCstatus")
+                    for name, item in input[f"{group_name}/SCstatus"].items():
+                        if isinstance(item, h5py.Dataset):
+                            shape = item.shape
                             g_sc.create_dataset(
                                 name,
-                                shape=(n_scans, n_pixels) + shape[2:],
-                                data=item[scans, :n_pixels],
-                            )
-                        else:
-                            g_sc.create_dataset(
-                                name, shape=(n_scans,), data=item[scans]
-                            )
-
-                g = output.create_group("S2")
-                for name, item in input["S2"].items():
-                    if isinstance(item, h5py.Dataset):
-                        shape = item.shape
-                        g.create_dataset(
-                            name,
-                            shape=(n_scans, n_pixels) + shape[2:],
-                            data=item[scans, :n_pixels],
-                        )
-
-                g_st = g.create_group("ScanTime")
-                for name, item in input["S2/ScanTime"].items():
-                    if isinstance(item, h5py.Dataset):
-                        shape = item.shape
-                        if len(shape) > 1:
-                            g_st.create_dataset(
-                                name,
-                                shape=(n_scans, n_pixels) + shape[2:],
-                                data=item[scans, :n_pixels],
-                            )
-                        else:
-                            g_st.create_dataset(
-                                name, shape=(n_scans,), data=item[scans]
-                            )
-
-                g_sc = g.create_group("SCstatus")
-                for name, item in input["S2/SCstatus"].items():
-                    if isinstance(item, h5py.Dataset):
-                        shape = item.shape
-                        if len(shape) > 1:
-                            g_sc.create_dataset(
-                                name,
-                                shape=(n_scans, n_pixels) + shape[2:],
-                                data=item[scans, :n_pixels],
-                            )
-                        else:
-                            g_sc.create_dataset(
-                                name, shape=(n_scans,), data=item[scans]
+                                shape=(n_scans,) + shape[1:],
+                                data=item[scan_range],
                             )
+                    i += 1
 
                 for a in input.attrs:
                     output.attrs[a] = input.attrs[a]
@@ -546,11 +484,11 @@ class L1CFile:
 
             # Handle case that observations are split up.
             tbs = []
-            tbs.append(input[f"{swath}/Tc"][indices])
+            tbs.append(input[f"{swath}/Tc"][:][indices])
             if "S2" in input.keys():
-                tbs.append(input["S2/Tc"][indices])
+                tbs.append(input["S2/Tc"][:][indices])
             if "S3" in input.keys():
-                tbs.append(input["S3/Tc"][indices])
+                tbs.append(input["S3/Tc"][:][indices])
 
             n_pixels = max([array.shape[1] for array in tbs])
             tbs_r = []
