@@ -42,6 +42,14 @@ LOGGER = logging.getLogger(__name__)
 # Struct types
 ###############################################################################
 
+CHANNEL_INDICES = {
+    "TMIPR": [0, 1, 2, 3, 4, 6, 7, 8, 9],
+    "TMIPO": [0, 1, 2, 3, 4, 6, 7, 8, 9],
+    "SSMI": [2, 3, 4, 6, 7, 8, 9],
+    "SSMIS": [2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14],
+    "AMSR2": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
+}
+
 N_SPECIES = 5
 N_TEMPERATURES = 12
 N_LAYERS = 28
@@ -342,11 +350,12 @@ class PreprocessorFile:
             for k, d in data.items():
                 d[i] = s[k]
 
-        if isinstance(self.sensor, sensors.ConstellationScanner):
+        if self.sensor.sensor_name in CHANNEL_INDICES:
+            ch_inds = CHANNEL_INDICES[self.sensor.sensor_name]
             tbs = data["brightness_temperatures"]
-            data["brightness_temperatures"] = tbs[..., self.sensor.gmi_channels]
+            data["brightness_temperatures"] = tbs[..., ch_inds]
             eia = data["earth_incidence_angle"]
-            data["earth_incidence_angle"] = eia[..., self.sensor.gmi_channels]
+            data["earth_incidence_angle"] = eia[..., ch_inds]
 
         dims = ["scans", "pixels", "channels"]
         data = {k: (dims[: len(d.shape)], d) for k, d in data.items()}
@@ -695,15 +704,19 @@ def has_preprocessor():
 
 # Dictionary mapping sensor IDs to preprocessor executables.
 PREPROCESSOR_EXECUTABLES = {
-    "GMI": "gprof2020pp_GMI_L1C",
-    "MHS": "gprof2020pp_MHS_L1C",
+    "GMI": "gprof2021pp_GMI_L1C",
+    "MHS": "gprof2021pp_MHS_L1C",
     "TMIPR": "gprof2021pp_TMI_L1C",
     "TMIPO": "gprof2021pp_TMI_L1C",
-    "SSMI": "gprof2020pp_SSMI_L1C",
-    ("GMI", "MHS"): "gprof2020pp_GMI_MHS_L1C",
-    ("GMI", "TMIPR"): "gprof2020pp_GMI_TMI_L1C",
-    ("GMI", "TMIPO"): "gprof2020pp_GMI_TMI_L1C",
-    ("GMI", "SSMI"): "gprof2020pp_GMI_SSMI_L1C"
+    "SSMI": "gprof2021pp_SSMI_L1C",
+    "SSMIS": "gprof2021pp_SSMIS_L1C",
+    "AMSR2": "gprof2021pp_AMSR2_L1C",
+    ("GMI", "MHS"): "gprof2021pp_GMI_MHS_L1C",
+    ("GMI", "TMIPR"): "gprof2021pp_GMI_TMI_L1C",
+    ("GMI", "TMIPO"): "gprof2021pp_GMI_TMI_L1C",
+    ("GMI", "SSMI"): "gprof2021pp_GMI_SSMI_L1C",
+    ("GMI", "SSMIS"): "gprof2021pp_GMI_SSMIS_L1C",
+    ("GMI", "AMSR2"): "gprof2021pp_GMI_AMSR2_L1C"
 }
 
 
