@@ -32,6 +32,7 @@ from gprof_nn.definitions import (
     LEVELS,
     DATABASE_MONTHS,
     PROFILE_NAMES,
+    SEAICE_YEARS
 )
 
 from gprof_nn.coordinates import latlon_to_ecef
@@ -63,6 +64,15 @@ GENERIC_HEADER = np.dtype(
 ###############################################################################
 
 
+CHANNEL_INDICES = {
+    "TMIPO": [0, 1, 2, 3, 4, 6, 7, 8, 9],
+    "TMIPR": [0, 1, 2, 3, 4, 6, 7, 8, 9],
+    "SSMI": [2, 3, 4, 6, 7, 8, 9],
+    "SSMIS": [2, 3, 4, 6, 7, 8, 9, 11, 12, 13, 14],
+    "AMSR2": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
+}
+
+
 class SimFile:
     """
     Interface class to read GPROF .sim files.
@@ -200,8 +210,9 @@ class SimFile:
             indices = np.clip(indices, 0, matched.shape[0] - 1)
 
             tbs = self.data["tbs_simulated"]
-            if isinstance(self.sensor, sensors.ConstellationScanner):
-                tbs = tbs[..., self.sensor.gmi_channels]
+            if self.sensor.sensor_name in CHANNEL_INDICES:
+                ch_inds = CHANNEL_INDICES[self.sensor.sensor_name]
+                tbs = tbs[..., ch_inds]
             # tbs = tbs.reshape((-1,) + shape[2:])
 
             matched[indices, ...] = tbs
@@ -225,8 +236,9 @@ class SimFile:
             matched[:] = np.nan
 
             biases = self.data["tbs_bias"]
-            if isinstance(self.sensor, sensors.ConstellationScanner):
-                biases = biases[..., self.sensor.gmi_channels]
+            if self.sensor.sensor_name in CHANNEL_INDICES:
+                ch_inds = CHANNEL_INDICES[self.sensor.sensor_name]
+                biases = biases[..., ch_inds]
             matched[indices, ...] = biases
 
             matched[indices, ...][dists > 10e3] = np.nan
@@ -315,16 +327,34 @@ class SimFile:
                 "tbs_bias",
                 "d_tbs",
             ]:
-                if isinstance(self.sensor, sensors.ConstellationScanner):
-                    data = data[..., self.sensor.gmi_channels]
+                if self.sensor.sensor_name in CHANNEL_INDICES:
+                    ch_inds = CHANNEL_INDICES[self.sensor.sensor_name]
+                    data = data[..., ch_inds]
 
             dims = ("samples",)
             if len(data.shape) > 1:
                 dims = dims + tuple([dim_dict[s] for s in data.shape[1:]])
 
-            results[key] = dims, self.data[key]
+            results[key] = dims, data
 
         dataset = xr.Dataset(results)
+
+        year = dataset["scan_time"].data["year"] - 1970
+        month = dataset["scan_time"].data["month"] - 1
+        day = dataset["scan_time"].data["day"] - 1
+        hour = dataset["scan_time"].data["hour"]
+        minute = dataset["scan_time"].data["minute"]
+        second = dataset["scan_time"].data["second"]
+        dates = (
+            year.astype("datetime64[Y]") +
+            month.astype("timedelta64[M]") +
+            day.astype("timedelta64[D]") +
+            hour.astype("timedelta64[h]") +
+            minute.astype("timedelta64[m]") +
+            second.astype("timedelta64[s]")
+        )
+        dataset["scan_time"] = (("samples",), dates)
+
         return dataset
 
 
@@ -406,16 +436,30 @@ def _extract_scenes(data):
 
     i_start = 0
     i_end = data.scans.size
+    i_step = n
+    j_end = data.pixels.size
+    j_step = min(data.pixels.size, n)
 
     scenes = []
-    while i_start + n < i_end:
-        subscene = data[{"scans": slice(i_start, i_start + n)}]
-        surface_precip = subscene["surface_precip"].data
-        if np.isfinite(surface_precip).sum() > 100:
-            scenes.append(subscene)
-            i_start += n
+    while i_start + i_step <= i_end:
+        j_start = 0
+        had_scene = False
+        while j_start + j_step <= j_end:
+            subscene = data[{
+                "scans": slice(i_start, i_start + n),
+                "pixels": slice(j_start, j_start + n)
+            }]
+            surface_precip = subscene["surface_precip"].data
+            if np.isfinite(surface_precip).sum() > 50:
+                scenes.append(subscene)
+                j_start += j_step
+                had_scene = True
+            else:
+                j_start += j_step // 2
+        if had_scene:
+            i_start += i_step
         else:
-            i_start += n // 2
+            i_start += i_step // 2
 
     if scenes:
         return xr.concat(scenes, "samples")
@@ -489,8 +533,8 @@ def _add_era5_precip(input_data, l1c_data, era5_data):
     l_0 = era5_data[{"longitude": slice(0, 1)}].copy(deep=True)
     l_0 = l_0.assign_coords({"longitude": [360.0]})
     era5_data = xr.concat([era5_data, l_0], "longitude")
-    n_scans = l1c_data.scans.size
-    n_pixels = l1c_data.pixels.size
+    n_scans = input_data.scans.size
+    n_pixels = input_data.pixels.size
 
     surface_types = input_data["surface_type"].data
     indices = (surface_types == 2) + (surface_types == 16)
@@ -500,7 +544,7 @@ def _add_era5_precip(input_data, l1c_data, era5_data):
     lons = np.where(lons < 0.0, lons + 360, lons)
     lons = xr.DataArray(lons, dims="samples")
     time = np.broadcast_to(
-        l1c_data["scan_time"].data.reshape(-1, 1), (n_scans, n_pixels)
+        input_data["scan_time"].data.reshape(-1, 1), (n_scans, n_pixels)
     )[indices]
     time = xr.DataArray(time, dims="samples")
     # Interpolate and convert to mm/h
@@ -565,7 +609,7 @@ def process_sim_file(sim_filename, sensor, configuration, era5_path, log_queue=N
     data_pp = run_preprocessor(
         l1c_file.filename, sensor=sensor, configuration=configuration, robust=False
     )
-    data_pp = data_pp.drop(
+    data_pp = data_pp.drop_vars(
         ["earth_incidence_angle", "sunglint_angle", "quality_flag",
          "wet_bulb_temperature", "lapse_rate"]
     )
@@ -658,7 +702,11 @@ def process_mrms_file(sensor, mrms_filename, configuration, day, log_queue=None)
     )
 
     scenes = []
-    LOGGER.debug("Found %s L1C file for MRMS file %s.", len(l1c_files), mrms_filename)
+    LOGGER.debug(
+        "Found %s L1C file for MRMS file %s.",
+        len(l1c_files),
+        mrms_filename
+    )
     for file in l1c_files:
         # Extract scans over CONUS ans run preprocessor.
         _, f_roi = tempfile.mkstemp()
@@ -671,14 +719,17 @@ def process_mrms_file(sensor, mrms_filename, configuration, day, log_queue=None)
             Path(f_roi).unlink()
         if data_pp is None:
             continue
+
         if data_pp.channels.size > sensor.n_chans:
-            data_pp = data_pp[{"channels": sensor.gmi_channels}]
+            if sensor.sensor_name in CHANNEL_INDICES:
+                ch_inds = CHANNEL_INDICES[sensor.sensor_name]
+                data_pp = data_pp[{"channels": ch_inds}]
 
         # Drop unneeded variables.
         drop = ["sunglint_angle", "quality_flag", "wet_bulb_temperature", "lapse_rate"]
         if not isinstance(sensor, sensors.CrossTrackScanner):
             drop.append("earth_incidence_angle")
-        data_pp = data_pp.drop(drop)
+        data_pp = data_pp.drop_vars(drop)
 
         LOGGER.debug("Matching MRMS data for %s.", file.filename)
         mrms_file.match_targets(data_pp)
@@ -697,6 +748,7 @@ def process_mrms_file(sensor, mrms_filename, configuration, day, log_queue=None)
         if new_scenes is not None:
             scenes.append(new_scenes)
 
+
     if scenes:
         dataset = xr.concat(scenes, "samples")
         dataset["source"] = (("samples",), np.ones(dataset.samples.size, dtype=np.int8))
@@ -726,18 +778,21 @@ def process_l1c_file(l1c_filename, sensor, configuration, era5_path, log_queue=N
     data_pp = run_preprocessor(
         l1c_filename, sensor=sensor, configuration=configuration, robust=False
     )
-
     l1c_file = L1CFile(l1c_filename)
-    if l1c_file.sensor != sensor:
-        data_pp = data_pp[{"channels": sensor.gmi_channels}]
+
     # Drop unneeded variables.
     drop = ["sunglint_angle", "quality_flag", "wet_bulb_temperature", "lapse_rate"]
     if not isinstance(sensor, sensors.CrossTrackScanner):
         drop.append("earth_incidence_angle")
-    data_pp = data_pp.drop(drop)
+    data_pp = data_pp.drop_vars(drop)
 
     if data_pp is None:
         return None
+    if data_pp.channels.size > sensor.n_chans:
+        if sensor.sensor_name in CHANNEL_INDICES:
+            ch_inds = CHANNEL_INDICES[self.sensor.sensor_name]
+            data_pp = data_pp[{"channels": ch_inds}]
+
     data_pp = add_targets(data_pp, sensor)
 
     l1c_data = L1CFile(l1c_filename).to_xarray_dataset()
@@ -753,9 +808,6 @@ def process_l1c_file(l1c_filename, sensor, configuration, era5_path, log_queue=N
     for v in ["surface_precip", "convective_precip"]:
         data_pp[v].data[~sea_ice] = np.nan
 
-    if data_pp.channels.size > sensor.n_chans:
-        data_pp = data_pp[{"channels": sensor.gmi_channels}]
-
     scenes = _extract_scenes(data_pp)
     scenes["source"] = (("samples",), 2 * np.ones(scenes.samples.size, dtype=np.int8))
     if scenes is not None:
@@ -978,12 +1030,24 @@ class SimFileProcessor:
 
         l1c_file_path = self.sensor.l1c_file_path
         l1c_files = []
-        for year, month in DATABASE_MONTHS:
-            try:
-                date = datetime(year, month, self.day)
-                l1c_files += L1CFile.find_files(date, l1c_file_path, sensor=self.sensor)
-            except ValueError:
-                pass
+
+        # Get L1C for specific year ...
+        if self.sensor.name in SEAICE_YEARS:
+            year = SEAICE_YEARS[self.sensor.name]
+            for month in range(1, 13):
+                try:
+                    date = datetime(year, month, self.day)
+                    l1c_files += L1CFile.find_files(date, l1c_file_path, sensor=self.sensor)
+                except ValueError:
+                    pass
+        # Or database period
+        else:
+            for year, month in DATABASE_MONTHS:
+                try:
+                    date = datetime(year, month, self.day)
+                    l1c_files += L1CFile.find_files(date, l1c_file_path, sensor=self.sensor)
+                except ValueError:
+                    pass
 
         # If no L1C files are found use GMI co-locations.
         if len(l1c_files) < 1:
