@@ -21,10 +21,21 @@ import xarray as xr
 from quantnn.normalizer import MinMaxNormalizer
 
 from gprof_nn import sensors
-from gprof_nn.data.utils import load_variable, decompress_scene, remap_scene
-from gprof_nn.definitions import MASKED_OUTPUT
+from gprof_nn.data.utils import (apply_limits,
+                                 compressed_pixel_range,
+                                 load_variable,
+                                 decompress_scene,
+                                 remap_scene,
+                                 upsample_scans)
+from gprof_nn.utils import expand_tbs
+from gprof_nn.definitions import (MASKED_OUTPUT,
+                                  LAT_BINS,
+                                  TIME_BINS,
+                                  LIMITS,
+                                  ALL_TARGETS)
 from gprof_nn.data.preprocessor import PreprocessorFile
-from gprof_nn.augmentation import get_transformation_coordinates
+from gprof_nn.augmentation import (get_transformation_coordinates,
+                                   extract_domain)
 
 LOGGER = logging.getLogger(__name__)
 
@@ -47,21 +58,24 @@ _INPUT_DIMENSIONS = {
     "TMIPR": (96, 128),
     "TMIPO": (96, 128),
     "SSMI": (96, 128),
+    "SSMIS": (32, 128),
+    "AMSR2": (32, 128),
     "MHS": (24, 128),
 }
 
 
-def calculate_resampling_indices(latitudes, sensor):
+def calculate_resampling_indices(latitudes, time, sensor):
     """
-    Calculate scene indices based on latitude distributions.
+    Calculate scene indices based on latitude and local times.
 
     Args:
         latitudes: Central latitudes of the scenes.
-        sensor: The sensor object to whose latitude distribution
-            to resample the scenes.
+        local_time: Time of day in minuts for each sample.
+        sensor: The sensor object to whose latitude and local
+            time sampling to to resample the scenes.
 
     Return:
-        None is the provided sensors has no latitude ratios
+        None if the provided sensor has no latitude ratios
         attribute. Otherwise an array of scene indices that
         resamples the scenes to match the latitude distribution
         of the sensor.
@@ -70,8 +84,15 @@ def calculate_resampling_indices(latitudes, sensor):
     if latitude_ratios is None:
         return None
 
-    indices = np.digitize(latitudes, np.linspace(-90, 90, 181)[1:-1])
-    weights = latitude_ratios[indices]
+    lat_indices = np.digitize(latitudes, LAT_BINS[1:-1])
+    time_indices = np.digitize(time, TIME_BINS[1:-1])
+
+    if latitude_ratios.ndim == 1:
+        weights = latitude_ratios[lat_indices]
+    else:
+        weights = latitude_ratios[lat_indices, time_indices]
+    weights = np.nan_to_num(weights, 0.0)
+    print(weights.sum())
     indices = np.arange(latitudes.size)
     probs = weights / weights.sum()
     return np.random.choice(indices, size=latitudes.size, p=probs)
@@ -537,9 +558,17 @@ class GPROF_NN_1D_Dataset(Dataset1DBase):
             else:
                 self.sensor = sensor
 
-
         latitudes = self.dataset.latitude.mean(("scans", "pixels")).data
-        indices = calculate_resampling_indices(latitudes, self.sensor)
+        longitudes = self.dataset.longitude.mean(("scans", "pixels")).data
+        if "pixels" in self.dataset.scan_time.dims:
+            scan_time = self.dataset.scan_time.mean(("scans", "pixels"))
+        else:
+            scan_time = self.dataset.scan_time.mean(("scans",))
+        local_time = (
+            scan_time + (longitudes / 360 * 24 * 60 * 60).astype("timedelta64[s]")
+        )
+        minutes = local_time.dt.hour.data * 60 + local_time.dt.minute.data
+        indices = calculate_resampling_indices(latitudes, minutes, self.sensor)
         if indices is None:
             kwargs = {}
         else:
@@ -810,7 +839,13 @@ class GPROF_NN_3D_Dataset:
             width, height = input_dimensions
 
         latitudes = self.dataset.latitude.mean(("scans", "pixels")).data
-        indices = calculate_resampling_indices(latitudes, self.sensor)
+        longitudes = self.dataset.longitude.mean(("scans", "pixels")).data
+        scan_time = self.dataset.scan_time.mean(("scans"))
+        local_time = (
+            scan_time + (longitudes / 360 * 24 * 60 * 60).astype("timedelta64[s]")
+        )
+        minutes = local_time.dt.hour.data * 60 + local_time.dt.minute.data
+        indices = calculate_resampling_indices(latitudes, minutes, self.sensor)
         if indices is None:
             kwargs = {}
         else:
@@ -1222,3 +1257,222 @@ class SimulatorDataset(GPROF_NN_3D_Dataset):
             y[k] = np.stack(y[k])
 
         return x, y
+
+
+HR_TARGETS = [
+    "surface_precip",
+    "rain_water_path",
+    "ice_water_path",
+    "cloud_water_path",
+    "rain_water_content",
+    "snow_water_content",
+    "cloud_water_content",
+    "latent_heat",
+]
+
+
+def _remap_scene(scene, coords, targets):
+    variables = ["brightness_temperatures",] + targets
+    data = {}
+
+    dims = ("scans", "pixels")
+    dims_3 = ("scans_3", "pixels")
+
+    i_start, _ = compressed_pixel_range()
+    coords_3 = upsample_scans(coords, axis=1)
+    coords_3[1] -= i_start
+    n_scans = scene.scans.size
+    scaling = (3 * n_scans - 2) / n_scans
+    coords_3[0] *= scaling
+
+    for v in variables:
+        if v in HR_TARGETS:
+            remap_coords = coords_3
+            dims = ("scans_3", "pixels")
+        else:
+            remap_coords = coords
+            dims = ("scans", "pixels")
+
+        data_v = scene[v].data
+        if v in LIMITS:
+            data_v = apply_limits(data_v, *LIMITS[v])
+            data_r = extract_domain(data_v, remap_coords, order=1)
+            data_r = data_r.astype(np.float32)
+            data[v] = (dims + scene[v].dims[2:], data_r)
+        else:
+            data[v] = (scene[v].dims, scene[v].data)
+
+    return xr.Dataset(data)
+
+
+class GPROF_NN_HR_Dataset(GPROF_NN_3D_Dataset):
+    """
+    Dataset to tran a neural network model on high resolution output
+    data.
+    """
+    def __init__(
+        self,
+        filename,
+        batch_size=32,
+        normalize=True,
+        normalizer=None,
+        transform_zeros=True,
+        shuffle=True,
+        augment=True,
+        targets=None
+    ):
+        """
+        Args:
+            filename: Path to the NetCDF file containing the training data.
+            batch_size: Number of samples in each training batch.
+            normalize: Whether or not to normalize the input data.
+            normalizer: The normalizer used to normalize the data.
+            transform_zeros: Whether or not to transform zeros to small
+                random values.
+            shuffle: Whether or not to shuffle the training data.
+            augment: Whether or not to randomly mask high-frequency channels
+                and to randomly permute ancillary data.
+            targets: List of targets to load.
+        """
+        self.filename = Path(filename)
+        # Load and decompress data but keep only scenes for which
+        # contain simulated obs.
+        self.dataset = decompress_and_load(self.filename)
+
+        if targets is None:
+            targets = HR_TARGETS
+
+        self.transform_zeros = transform_zeros
+        self.batch_size = batch_size
+        self.shuffle = shuffle
+        self.augment = augment
+
+        seed = int.from_bytes(os.urandom(4), "big") + os.getpid()
+        self._rng = np.random.default_rng(seed)
+
+        x, y = self.load_training_data_3d(self.dataset, targets, augment, self._rng)
+        if normalizer is None:
+            self.normalizer = MinMaxNormalizer(x)
+        elif isinstance(normalizer, type):
+            self.normalizer = normalizer(x)
+        else:
+            self.normalizer = normalizer
+
+        self.normalize = normalize
+        if normalize:
+            x = self.normalizer(x)
+
+        self.x = x
+        self.y = y
+
+        self.x = self.x.astype(np.float32)
+        if isinstance(self.y, dict):
+            self.y = {k: self.y[k].astype(np.float32) for k in self.y}
+        else:
+            self.y = self.y.astype(np.float32)
+
+        self._shuffled = False
+        self.indices = np.arange(self.x.shape[0])
+        if self.shuffle:
+            self._shuffle()
+
+        # Delete decompressed raw input data.
+        del self.dataset
+
+
+    def load_training_data_3d(self, dataset, targets, augment, rng):
+        """
+        Load data for training a simulator data.
+
+        This function is a replacement for the ``load_training_data3d``
+        method of the sensor that is called by the other training data
+        objects to load the data. This is required because the data
+        the input for the simulator are always the GMI Tbs.
+
+        Args:
+            dataset: The 'xarray.Dataset' from which to load the training
+                data.
+            targets: List of the targets to load.
+            augment: Whether or not to augment the training data.
+            rng: 'numpy.random.Generator' to use to generate random numbers.
+
+        Return:
+
+            Tuple ``(x, y)`` containing the training input ``x`` and a
+            dictionary of target data ``y``.
+        """
+        # Brightness temperatures
+        n = dataset.samples.size
+
+        x = []
+        y = {}
+
+        vs = ["latitude", "longitude"]
+
+        for i in range(n):
+
+            scene = dataset[{"samples": i}]
+
+            if augment:
+                p_x_o = rng.random()
+                p_x_i = rng.random()
+                p_y = rng.random()
+            else:
+                p_x_o = 0.5
+                p_x_i = 0.5
+                p_y = 0.5
+
+            lats = scene.latitude.data
+            lons = scene.longitude.data
+            geometry = sensors.GMI.viewing_geometry
+            coords = get_transformation_coordinates(
+                lats, lons, geometry, 96, 128, p_x_i, p_x_o, p_y
+            )
+
+            scene = _remap_scene(scene, coords, targets + vs)
+
+            #
+            # Input data
+            #
+
+            tbs = sensors.GMI.load_brightness_temperatures(scene)
+            tbs = expand_tbs(tbs)
+            tbs = np.transpose(tbs, (2, 0, 1))
+            if augment:
+                r = rng.random()
+                n_p = rng.integers(10, 30)
+                if r > 0.80:
+                    tbs[10:15, :, :n_p] = np.nan
+            x.append(tbs)
+
+            #
+            # Output data
+            #
+
+            for t in targets:
+                y_t = sensors.GMI.load_target(scene, t, None)
+                y_t = np.nan_to_num(y_t, nan=MASKED_OUTPUT)
+                dims_sp = tuple(range(2))
+                dims_t = tuple(range(2, y_t.ndim))
+
+                y.setdefault(t, []).append(np.transpose(y_t, dims_t + dims_sp))
+
+            # Also flip data if requested.
+            if augment:
+                r = rng.random()
+                if r > 0.5:
+                    x[i] = np.flip(x[i], -2)
+                    for k in targets:
+                        y[k][i] = np.flip(y[k][i], -2)
+
+                r = rng.random()
+                if r > 0.5:
+                    x[i] = np.flip(x[i], -1)
+                    for k in targets:
+                        y[k][i] = np.flip(y[k][i], -1)
+
+        x = np.stack(x)
+        for k in targets:
+            y[k] = np.stack(y[k])
+
+        return x, y
