@@ -37,7 +37,9 @@ _BASE_URL = "https://pmm-gv.gsfc.nasa.gov/pub/NMQ/level2/"
 
 PATHS = {
     "GMI": "GPM/",
-    "TMIPO": "TRMM/"
+    "TMIPO": "TRMM/",
+    "SSMIS": "F17/",
+    "AMSR2": "GCOMW1/",
 }
 
 LINK_REGEX = re.compile(r"<a href=\"([\w\.]*)\">")
@@ -130,7 +132,6 @@ def open_validation_data(files):
     rqi = np.zeros((len(times), n_rows, n_cols), dtype=np.float32)
     for i, f in enumerate(rqi_files):
         rqi[i, :, :] = np.loadtxt(f, skiprows=6, dtype=np.float32)
-    print(rqi.max())
     rqi[rqi < 0.0] = np.nan
     data["radar_quality_index"] = (dims, rqi)
 
@@ -283,17 +284,16 @@ def unify_grid(latitude, longitude):
 
 def calculate_angles(l1c_data):
     """
-    For L1C data from a conical scanner, calculate the angle that each
-    footprint in the scan is rotated with respect to along-track
-    direction.
+    Calculate the rotation angle of each footprint with respect to the
+    along-track direction.
 
     Args:
         l1c_data: xarray.Dataset containing the L1C data.
 
     Return:
-        2D array containing the angles between local along-track direction
-        and the projection of the viewing vector onto the horizontal local
-        horizontal plane.
+        2D array containing the angles between the along-track direction and
+        the projection of the viewing vector onto the local tangent plane of
+        the Earth.
     """
     lats = l1c_data.latitude
     lats_sc = l1c_data.spacecraft_latitude
@@ -312,12 +312,15 @@ def calculate_angles(l1c_data):
     alt_sc = alt_sc.data
 
     t = augmentation.latlon_to_ecef()
+    # Earth-centered earth-fixed (ECEF) coordinats at surface.
     xyz = np.stack(t.transform(
         lons, lats, np.zeros_like(lons), radians=False
     ), axis=-1)
+    # ECEF coordinates at 1m, used to derive local up vector.
     xyz_1 = np.stack(t.transform(
         lons, lats, np.ones_like(lons), radians=False
     ), axis=-1)
+    # ECEF coordinates of space craft.
     xyz_sc = np.stack(t.transform(
         lons_sc, lats_sc, alt_sc, radians=False
     ), axis=-1)
@@ -325,6 +328,7 @@ def calculate_angles(l1c_data):
     # Calculate local along track vector
     a = xyz[1:] - xyz[:-1]
     a /= np.sqrt((a ** 2).sum(axis=-1, keepdims=True))
+    # Extend center points to GPM pixels.
     a_e = np.zeros((a.shape[0] + 1, a.shape[1], 3))
     a_e[:-1] += 0.5 * a
     a_e[1:] += 0.5 * a
@@ -336,8 +340,10 @@ def calculate_angles(l1c_data):
     z = xyz_1 - xyz
     z = z / np.sqrt((z ** 2).sum(axis=-1, keepdims=True))
     x = np.cross(z, a)
-
     v = xyz - xyz_sc
+
+    # Calculate projection components in tangent plane
+    # and return angle.
     v_p_a = (v * a).sum(axis=-1)
     v_p_x = (v * x).sum(axis=-1)
     return np.rad2deg(np.arctan2(v_p_x, v_p_a))
@@ -513,18 +519,20 @@ class ValidationFileProcessor:
         # Set pixel with too few valid neighboring pixels to nan.
         sp[sp_cts < 1e-1] = np.nan
 
-        surface_precip.data[:] = sp
         surface_precip = surface_precip.interp(
-            latitude=lats_5, longitude=lons_5, time=time
+            latitude=lats_5, longitude=lons_5, time=time, method="nearest",
+                kwargs={"fill_value": np.nan}
         )
         datasets = [surface_precip]
         rqi = mrms_data.radar_quality_index.interp(
-            latitude=lats_5, longitude=lons_5, time=time, method="nearest"
+            latitude=lats_5, longitude=lons_5, time=time, method="nearest",
+            kwargs={"fill_value": np.nan}
         )
         datasets.append(rqi)
         if "mask" in mrms_data.variables:
             mask = mrms_data.mask.interp(
-                latitude=lats_5, longitude=lons_5, time=time, method="nearest"
+                latitude=lats_5, longitude=lons_5, time=time, method="nearest",
+                kwargs={"fill_value": -1}
             )
             datasets.append(mask)
         mrms_data = xr.merge(datasets)
@@ -570,11 +578,21 @@ class ValidationFileProcessor:
         mrms_file = Path(mrms_file)
         preprocessor_file = Path(preprocessor_file)
 
-        l1c_file = L1CFile.open_granule(granule, self.sensor.l1c_file_path, self.sensor)
+        l1c_file = L1CFile.open_granule(
+            granule,
+            self.sensor.l1c_file_path,
+            self.sensor
+        )
         with TemporaryDirectory() as tmp:
             tmp = Path(tmp)
             l1c_sub_file = tmp / "l1c_file.HDF5"
-            scan_start, scan_end = l1c_file.extract_scans(CONUS, l1c_sub_file)
+            scan_start, scan_end = l1c_file.extract_scans(
+                CONUS,
+                l1c_sub_file,
+                min_scans=256
+            )
+            if scan_end - scan_start <= 0:
+                return None
 
             # Extract reference data from MRMS file.
             if not mrms_file.exists():
