@@ -23,6 +23,7 @@ GPM constellation. Most sensors exist in two or three variants:
 """
 from abc import ABC, abstractmethod, abstractproperty
 from concurrent.futures import ProcessPoolExecutor
+from copy import copy
 from pathlib import Path
 
 import numpy as np
@@ -108,12 +109,12 @@ def calculate_smoothing_kernels(sensor):
     res_a_source = 13.5e3
     angles = sensor.angles
     res_x_target = geometry.get_resolution_x(angles)
-    res_a_target = geometry.get_resolution_a()
+    res_a_target = geometry.get_resolution_a(angles)
 
     kernels = []
-    for res in res_x_target:
+    for res_a, res_x in zip(res_a_target, res_x_target):
         k = calculate_smoothing_kernel(
-            res_x_source, res_a_source, res, res_a_target, size=11
+            res_x_source, res_a_source, res_x, res_a, size=11
         )
         kernels.append(k)
     return kernels
@@ -186,8 +187,10 @@ class Sensor(ABC):
         n_angles = len(angles)
         self._n_chans = n_chans
         self._n_angles = n_angles
+        self._latitude_ratios = None
         self.platform = platform
         self.viewing_geometry = viewing_geometry
+        self.missing_channels = None
 
         # Bin file types
         self._bin_file_header = types.get_bin_file_header(n_chans, n_angles, kind)
@@ -409,6 +412,16 @@ class Sensor(ABC):
         """
         pass
 
+    @property
+    def latitude_ratios(self):
+        """
+        Latitude ratios to use for resampling of training data.
+        """
+        if self._latitude_ratios is not None:
+            if not isinstance(self._latitude_ratios, np.ndarray):
+                self._latitude_ratios = np.load(self._latitude_ratios)
+        return self._latitude_ratios
+
     def load_training_data_3d(self, filename, targets):
         """
         Load training data for GPROF-NN 3D algorithm from NetCDF file.
@@ -675,9 +688,17 @@ class ConicalScanner(Sensor):
 
         lats = scene.latitude.data
         lons = scene.longitude.data
+        if augment:
+            alt_old = copy(self.viewing_geometry.altitude)
+            so_old = copy(self.viewing_geometry.scan_offset)
+            self.viewing_geometry.altitude = alt_old + np.random.uniform(-10, 10)
+            self.viewing_geometry.scan_offset = so_old + np.random.uniform(-0.2, 0.2)
         coords = get_transformation_coordinates(
             lats, lons, self.viewing_geometry, width, height, p_x_i, p_x_o, p_y
         )
+        if augment:
+            self.viewing_geometry.altitude = alt_old
+            self.viewing_geometry.scan_offset = so_old
 
         scene = remap_scene(scene, coords, targets)
 
@@ -852,9 +873,15 @@ class ConstellationScanner(ConicalScanner):
 
         self.modeling_error = modeling_error
 
-        # Calculate scence resampling factors
         self._latitude_ratios = latitude_ratios
 
+        # MRMS types
+        self._mrms_file_record = types.get_mrms_file_record(
+            self.n_chans,
+            self.n_angles,
+            types.CONICAL_CONST
+        )
+
     @property
     def correction(self):
         if self._correction is not None:
@@ -862,13 +889,6 @@ class ConstellationScanner(ConicalScanner):
                 self._correction = CdfCorrection(self._correction)
         return self._correction
 
-    @property
-    def latitude_ratios(self):
-        if self._latitude_ratios is not None:
-            if not isinstance(self._latitude_ratios, np.ndarray):
-                self._latitude_ratios = np.load(self._latitude_ratios)
-        return self._latitude_ratios
-
     def load_brightness_temperatures(self, data, mask=None):
         """
         Load and bias-correct simulated brightness temperatures.
@@ -967,7 +987,7 @@ class ConstellationScanner(ConicalScanner):
                 )
             else:
                 tbs = scene["brightness_temperatures"].data
-                mask = mask * np.all((tbs > 0) * (tbs < 500), axis=-1)
+                mask = mask * np.any((tbs > 0) * (tbs < 500), axis=-1)
 
             #
             # Input data
@@ -981,12 +1001,21 @@ class ConstellationScanner(ConicalScanner):
             if self.correction is not None:
                 tbs = self.correction(self, st, None, tcwv, tbs, augment=augment)
 
+            # Add thermal noise to simulated observations.
             if augment and self.nedt is not None:
                 noise = rng.normal(size=tbs.shape)
                 for i in range(noise.shape[-1]):
                     noise[..., i] *= self.nedt[i]
                 tbs = tbs + noise
 
+            # Randomly set missing channels to missing.
+            if self.missing_channels is not None:
+                p_thresh = 1 / len(self.missing_channels)
+                for channel in self.missing_channels:
+                    tbs_c = tbs[..., channel]
+                    p = rng.uniform(size=tbs_c.shape)
+                    tbs_c[p <= p_thresh] = np.nan
+
             st = self.load_surface_type(scene, mask=mask)
             t2m = t2m[..., np.newaxis]
             tcwv = tcwv[..., np.newaxis]
@@ -1079,6 +1108,8 @@ class ConstellationScanner(ConicalScanner):
                 for i, n in enumerate(noise):
                     tbs[..., i] += self.modeling_error[i] * n
 
+            # Simulate missing obs at edge of swath.
+            # This may not be not necessary for concical scanners.
             r = rng.random()
             if r > 0.80:
                 n_p = rng.integers(6, 16)
@@ -1088,6 +1119,17 @@ class ConstellationScanner(ConicalScanner):
                 n_p = rng.integers(6, 16)
                 tbs[:, -n_p:] = np.nan
 
+            # Randomly set missing channels to missing.
+            if augment:
+                p = rng.uniform()
+                if p > 0.75:
+                    if self.missing_channels is not None:
+                        p_thresh = 0.5
+                        for channel in self.missing_channels:
+                            p = rng.uniform()
+                            if p <= p_thresh:
+                                tbs[..., channel] = np.nan
+
         tcwv = tcwv[np.newaxis]
         t2m = t2m[np.newaxis]
         st = self.load_surface_type(scene)
@@ -1119,7 +1161,6 @@ class ConstellationScanner(ConicalScanner):
             r = rng.random()
             if r > 0.5:
                 x = np.flip(x, -2)
-                x[self.n_chans] *= -1.0
                 for k in targets:
                     y[k] = np.flip(y[k], -2)
 
@@ -1171,16 +1212,21 @@ class ConstellationScanner(ConicalScanner):
 
         tbs = self.load_brightness_temperatures(scene, None)
         tbs = tbs[i_start:i_end, j_start:j_end]
+
+        # Randomly set missing channels to missing.
+        if augment:
+            p = rng.uniform()
+            if p > 0.75:
+                if self.missing_channels is not None:
+                    p_thresh = 0.5
+                    for channel in self.missing_channels:
+                        p = rng.uniform()
+                        if p <= p_thresh:
+                            tbs[..., channel] = np.nan
+
+        # Move channel to first dim.
         tbs = np.transpose(tbs, (2, 0, 1))
 
-        r = rng.random()
-        if r > 0.80:
-            n_p = rng.integers(6, 16)
-            tbs[:, :n_p] = np.nan
-        r = rng.random()
-        if r > 0.80:
-            n_p = rng.integers(6, 16)
-            tbs[:, -n_p:] = np.nan
 
         t2m = self.load_two_meter_temperature(scene)
         t2m = t2m[np.newaxis, i_start:i_end, j_start:j_end]
@@ -1250,7 +1296,7 @@ class ConstellationScanner(ConicalScanner):
             vs += ["surface_precip"]
 
         if indices is None:
-            indinces = range(n)
+            indices = range(n)
         for sample_index in indices:
             scene = decompress_scene(dataset[{"samples": sample_index}], targets + vs)
             source = scene.source
@@ -1315,7 +1361,7 @@ class CrossTrackScanner(Sensor):
             self._correction = correction
         # Convert modeling error to list of channel-wise modeling errors.
         if isinstance(modeling_error, int):
-            modeling_error = [modelling_error] * n_chans
+            modeling_error = [modeling_error] * n_chans
         self.modeling_error = modeling_error
 
     def __repr__(self):
@@ -1406,7 +1452,7 @@ class CrossTrackScanner(Sensor):
                 v = v[..., 0]
         return v
 
-    def load_training_data_1d(self, dataset, targets, augment, rng):
+    def load_training_data_1d(self, dataset, targets, augment, rng, indices=None):
         """
         Load training data for GPROF-NN 1D retrieval. This function will
         only load pixels that with a finite surface precip value in order
@@ -1457,9 +1503,12 @@ class CrossTrackScanner(Sensor):
         if "surface_precip" not in targets:
             vs += ["surface_precip"]
 
-        for i in range(n_samples):
-            scene = decompress_scene(dataset[{"samples": i}], targets + vs)
-            source = dataset.source[i]
+        if indices is None:
+            indices = range(n_samples)
+
+        for sample_index in indices:
+            scene = decompress_scene(dataset[{"samples": sample_index}], targets + vs)
+            source = dataset.source[sample_index]
 
             sp = scene["surface_precip"].data
             mask = np.all(sp >= 0, axis=-1)
@@ -1490,9 +1539,7 @@ class CrossTrackScanner(Sensor):
                 weights = None
                 tbs = scene["brightness_temperatures"].data
                 mask = mask * np.all((tbs > 0) * (tbs < 500), axis=-1)
-                eia = load_variable(scene, "earth_incidence_angle", mask=mask)[
-                    ..., 0
-                ]
+                eia = load_variable(scene, "earth_incidence_angle", mask=mask)
 
             #
             # Input data
@@ -1721,7 +1768,7 @@ class CrossTrackScanner(Sensor):
             tbs[:, -n_p:] = np.nan
 
         eia = self.load_earth_incidence_angle(scene)
-        eia = eia[np.newaxis, i_start:i_end, j_start:j_end, 0]
+        eia = eia[np.newaxis, i_start:i_end, j_start:j_end]
 
         t2m = self.load_two_meter_temperature(scene)
         t2m = t2m[np.newaxis, i_start:i_end, j_start:j_end]
@@ -1839,9 +1886,16 @@ class Platform:
         return f"Platform(name={self.name})"
 
 
-TRMM = Platform("TRMM", "/pdata4/archive/GPM/1C_TMI/", "1C.TRMM.TMI")
-NOAA19 = Platform("NOAA19", "/pdata4/archive/GPM/1C_NOAA19/", "1C.NOAA19.MHS")
-GPM = Platform("GPM-CO", "/pdata4/archive/GPM/1CR_GMI/", "1C-R.GPM.GMI")
+TRMM = Platform("TRMM", "/pdata4/archive/GPM/1C_TMI_ITE/", "1C.TRMM.TMI")
+NOAA19 = Platform("NOAA19", "/pdata4/archive/GPM/1C_NOAA19_ITE/", "1C.NOAA19.MHS")
+GPM = Platform("GPM-CO", "/pdata4/archive/GPM/1CR_GMI_ITE/", "1C-R.GPM.GMI")
+F15 = Platform("F15", "/pdata4/archive/GPM/1C_F15_ITE/", "1C.F15.SSMI")
+F17 = Platform("F17", "/pdata4/archive/GPM/1C_F17_ITE/", "1C.F17.SSMIS")
+GCOMW1 = Platform(
+    "GCOM-W1",
+    "/pdata4/archive/GPM/1C_AMSR2_ITE/",
+    "1C.GCOMW1.AMSR2"
+)
 
 ###############################################################################
 # GMI
@@ -1865,6 +1919,7 @@ GMI_CHANNELS = [
     (190.0, "H"),
 ]
 
+
 GMI_ANGLES = np.array([
     52.8,
     52.8,
@@ -1885,11 +1940,11 @@ GMI_ANGLES = np.array([
 
 
 GMI_VIEWING_GEOMETRY = Conical(
-    altitude=455e3,
+    altitude=444e3,
     earth_incidence_angle=53.0,
     scan_range=140.0,
     pixels_per_scan=221,
-    scan_offset=13.4e3,
+    scan_offset=13.2e3,
 )
 
 
@@ -1923,7 +1978,11 @@ MHS_CHANNELS = [
 MHS_NEDT = np.array([0.3, 0.5, 3.8, 0.7, 0.3])
 
 MHS_VIEWING_GEOMETRY = CrossTrack(
-    altitude=855e3, scan_range=2.0 * 49.5, pixels_per_scan=90, scan_offset=17e3
+    altitude=855e3,
+    scan_range=2.0 * 49.5,
+    pixels_per_scan=90,
+    scan_offset=17e3,
+    beam_width=1.1
 )
 
 MHS_GMI_CHANNELS = [8, 11, 13, 13, 14]
@@ -1939,6 +1998,8 @@ MHS = CrossTrackScanner(
     "MHS.dbsatTb.??????{day}.??????.sim",
     "/qdata1/pbrown/dbaseV7/simV7x",
     MHS_GMI_CHANNELS,
+    correction=DATA_FOLDER / "corrections_mhs.nc",
+    modeling_error=[3.0, 2.0, 2.0, 2.0, 2.0]
 )
 
 MHS_NOAA19 = CrossTrackScanner(
@@ -2141,14 +2202,24 @@ SSMI_NEDT = np.array([
  	0.31,
 ])
 
+SSMI_MODELING_ERROR = np.sqrt(np.array([
+    1.0,
+    1.5,
+    0.5,
+    1.0,
+    2.0,
+    1.5,
+    2.0,
+]))
+
 SSMI_GMI_CHANNELS = [2, 3, 4, 6, 7, 8, 9]
 
 SSMI_VIEWING_GEOMETRY = Conical(
-    altitude=350e3,
+    altitude=833e3,
     earth_incidence_angle=53.0,
-    scan_range=130.0,
-    pixels_per_scan=208,
-    scan_offset=13.4e3,
+    scan_range=102.4,
+    pixels_per_scan=128,
+    scan_offset=12.5e3,
 )
 
 SSMI = ConstellationScanner(
@@ -2156,11 +2227,219 @@ SSMI = ConstellationScanner(
     SSMI_CHANNELS,
     SSMI_NEDT,
     SSMI_ANGLES,
-    TRMM,
+    F15,
     SSMI_VIEWING_GEOMETRY,
     None,
     "SSMI.dbsatTb.??????{day}.??????.sim",
     "/qdata1/pbrown/dbaseV7/simV7_ssmi",
-    SSMI_GMI_CHANNELS
+    SSMI_GMI_CHANNELS,
+    modeling_error=SSMI_MODELING_ERROR,
+    correction=DATA_FOLDER / "corrections_ssmi.nc",
 )
 
+SSMI_F08 = ConstellationScanner(
+    "SSMI",
+    SSMI_CHANNELS,
+    SSMI_NEDT,
+    SSMI_ANGLES,
+    F15,
+    SSMI_VIEWING_GEOMETRY,
+    None,
+    "SSMI.dbsatTb.??????{day}.??????.sim",
+    "/qdata1/pbrown/dbaseV7/simV7_ssmi",
+    SSMI_GMI_CHANNELS,
+    modeling_error=SSMI_MODELING_ERROR,
+    correction=DATA_FOLDER / "corrections_ssmi.nc",
+)
+SSMI_F08.missing_channels = [5, 6]
+
+###############################################################################
+# SSMIS
+###############################################################################
+
+SSMIS_CHANNELS = [
+    (19.35, "V"),
+    (19.35, "H"),
+    (22.235, "V"),
+    (37.0, "V"),
+    (37.0, "H"),
+    (91.655, "V"),
+    (91.655, "H"),
+    (150, "H"),
+    (189, "H"),
+    (186, "H"),
+    (184, "H")
+]
+
+SSMIS_ANGLES = np.array([
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+])
+
+SSMIS_NEDT = np.array([
+    0.7,
+ 	0.7,
+ 	0.7,
+ 	0.5,
+ 	0.5,
+ 	0.9,
+ 	0.9,
+ 	1.0,
+ 	1.0,
+ 	1.0,
+ 	1.0,
+])
+
+SSMIS_MODELING_ERROR = np.sqrt(np.array([
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+]))
+
+SSMIS_GMI_CHANNELS = [
+    2,
+    3,
+    4,
+    6,
+    7,
+    8,
+    9,
+    11,
+    14,
+    14,
+    13
+]
+
+
+SSMIS_VIEWING_GEOMETRY = Conical(
+    altitude=880e3,
+    earth_incidence_angle=53.0,
+    scan_range=140,
+    pixels_per_scan=180,
+    scan_offset=12.5e3,
+)
+
+SSMIS = ConstellationScanner(
+    "SSMIS",
+    SSMIS_CHANNELS,
+    SSMIS_NEDT,
+    SSMIS_ANGLES,
+    F17,
+    SSMIS_VIEWING_GEOMETRY,
+    "/pdata4/veljko/SSMIS2MRMS_match2019/monthly_2021/",
+    "SSMIS.dbsatTb.??????{day}.??????.sim",
+    "/qdata1/pbrown/dbaseV7/simV7_ssmis",
+    SSMIS_GMI_CHANNELS,
+    modeling_error=SSMIS_MODELING_ERROR,
+    correction=DATA_FOLDER / "corrections_ssmis.nc",
+)
+
+SSMIS.missing_channels = [3, 7, 8, 9, 10]
+
+###############################################################################
+# AMSR2
+###############################################################################
+
+AMSR2_CHANNELS = [
+    (10.65, "V"),
+    (10.65, "H"),
+    (18.7, "V"),
+    (18.7, "H"),
+    (23.8, "V"),
+    (23.8, "H"),
+    (36.5, "V"),
+    (36.5, "H"),
+    (89, "V"),
+    (89, "H"),
+]
+
+AMSR2_ANGLES = np.array([
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+    53.1,
+])
+
+AMSR2_NEDT = np.array([
+    0.7,
+    0.7,
+    0.7,
+    0.7,
+    0.6,
+    0.6,
+    0.7,
+    0.7,
+    1.2,
+    1.4
+])
+
+AMSR2_MODELING_ERROR = np.sqrt(np.array([
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+    2.0,
+]))
+
+AMSR2_GMI_CHANNELS = [
+    0,
+    1,
+    2,
+    3,
+    4,
+    4,
+    6,
+    7,
+    8,
+    9
+]
+
+AMSR2_VIEWING_GEOMETRY = Conical(
+    altitude=700e3,
+    earth_incidence_angle=55.0,
+    scan_range=140,
+    pixels_per_scan=180,
+    scan_offset=12.5e3,
+)
+
+AMSR2 = ConstellationScanner(
+    "AMSR2",
+    AMSR2_CHANNELS,
+    AMSR2_NEDT,
+    AMSR2_ANGLES,
+    GCOMW1,
+    AMSR2_VIEWING_GEOMETRY,
+    "/pdata4/veljko/AMSR22MRMS_match2019/monthly_2021/",
+    "AMSR2.dbsatTb.??????{day}.??????.sim",
+    "/qdata1/pbrown/dbaseV7/simV7_amsr2",
+    AMSR2_GMI_CHANNELS,
+    modeling_error=AMSR2_MODELING_ERROR,
+    correction=DATA_FOLDER / "corrections_amsr2.nc",
+)
