@@ -24,7 +24,7 @@ import xarray as xr
 from rich.progress import track
 
 import gprof_nn.logging
-from gprof_nn.definitions import ALL_TARGETS
+from gprof_nn.definitions import ALL_TARGETS, LAT_BINS, TIME_BINS
 from gprof_nn.data.retrieval import RetrievalFile
 from gprof_nn.data.bin import BinFile
 from gprof_nn.data.preprocessor import PreprocessorFile
@@ -73,9 +73,69 @@ def open_file(filename):
     raise ValueError(f"Could not figure out how handle the file {filename.name}.")
 
 
-###############################################################################
-# Statistic interface
-###############################################################################
+def resample_scans(dataset, statistics):
+    """
+    Resample scans according to scan time and latitudes.
+
+    Args:
+        dataset: An xarray dataset containing observations or
+            retrieval results.
+        statistics: A 1 or 2D array containing resampling weights
+            to resample observations based on local time and
+            latitude.
+    """
+    if statistics.ndim == 1:
+        if statistics.size != LAT_BINS.size - 1:
+            raise ValueError(
+                "If statistics is 1D, its size must be equal to that of"
+                " definintion.LAT_BINS - 1."
+                )
+
+    if statistics.ndim == 2:
+        if statistics.shape[0] != LAT_BINS.size - 1:
+            raise ValueError(
+                "If statistics is 2D, its size along dim. 0 must be equal "
+                "to that of definintion.LAT_BINS - 1."
+            )
+        if statistics.shape[1] != TIME_BINS.size - 1:
+            raise ValueError(
+                "If statistics is 2D, its size along dim. 1 must be equal "
+                "to that of definintion.TIME_BINS - 1."
+            )
+
+
+    if "scan_time" not in dataset:
+        return dataset
+
+    longitude = dataset.longitude.mean("pixels")
+    latitude = dataset.latitude.mean("pixels")
+    scan_time = dataset.scan_time.mean("scans")
+    local_time = (
+        scan_time + (longitude / 360 * 24 * 60 * 60).astype("timedelta64[s]")
+    )
+    minutes = local_time.dt.hour * 60 + local_time.dt.minute.data
+
+    # Calculate resampling weights.
+    lat_indices = np.digitize(latitude.data, LAT_BINS[1:-1])
+    time_indices = np.digitize(minutes.data, TIME_BINS[1:-1])
+    if statistics.ndim == 1:
+        weights = statistics[lat_indices]
+    else:
+        weights = statistics[lat_indices, time_indices]
+
+    # Use weights to resample indices.
+    indices = np.arange(latitude.size)
+    norm = weights.sum()
+    # Return empty dataset if sum of weights is zero.
+    if np.isclose(norm, 0.0):
+        return dataset[{"scans": []}]
+
+    probs = weights / norm
+    indices = np.random.choice(indices, size=latitude.size, p=probs)
+
+
+    # Now resample scans.
+    return dataset[{"scans": indices}]
 
 
 class Statistic(ABC):
@@ -185,12 +245,13 @@ class ZonalDistribution(Statistic):
     results in either GPROF binary format or NetCDF format.
     """
 
-    def __init__(self, monthly=False):
+    def __init__(self, monthly=False, statistics=None):
         """
         Args:
             Name of the retrieval variable for which to compute the
             scan position mean.
         """
+        self.statistics = statistics
         self.latitude_bins = np.linspace(-90, 90, 181)
         self.surface_precip_bins = np.logspace(-2, 2.5, 201)
         self.has_time = monthly
@@ -307,6 +368,10 @@ class ZonalDistribution(Statistic):
         """
         self.sensor = sensor
         data = open_file(filename)
+
+        if self.statistics is not None:
+            data = resample_scans(data, self.statistics)
+
         if self.counts is None:
             self._initialize(data)
 
@@ -1040,11 +1105,17 @@ class TrainingDataStatistics(Statistic):
         self.t2m_bins = np.linspace(239.5, 339.5, 101)
         self.tcwv = np.zeros((18, 100), dtype=np.float32)
         self.tcwv_bins = np.linspace(-0.5, 99.5, 101)
-        self.lat_bins = np.linspace(-90, 90, 181)
+
         self.st = np.zeros(18, dtype=np.float32)
         self.at = np.zeros(4, dtype=np.float32)
+
+        self.lat_bins = LAT_BINS
         self.lats = np.zeros(180, dtype=np.float32)
 
+        self.time_bins = TIME_BINS
+        self.local_time = np.zeros(24, dtype=np.float32)
+        self.lat_local_time = np.zeros((180, 24), dtype=np.float32)
+
         self.sums_tcwv = {}
         self.counts_tcwv = {}
         self.has_angles = False
@@ -1111,7 +1182,7 @@ class TrainingDataStatistics(Statistic):
         if self.kind.upper() == "1D":
             dataset = GPROF_NN_1D_Dataset(
                 filename,
-                targets=ALL_TARGETS + ["latitude"],
+                targets=ALL_TARGETS + ["latitude", "longitude", "scan_time"],
                 normalize=False,
                 shuffle=False,
                 augment=False,
@@ -1121,7 +1192,7 @@ class TrainingDataStatistics(Statistic):
         elif self.kind.upper() == "3D":
             dataset = GPROF_NN_3D_Dataset(
                 filename,
-                targets=ALL_TARGETS + ["latitude"],
+                targets=ALL_TARGETS + ["latitude", "longitude", "scan_time"],
                 normalize=False,
                 shuffle=False,
                 augment=False,
@@ -1208,10 +1279,26 @@ class TrainingDataStatistics(Statistic):
         bins = np.arange(-1, 4) + 0.5
         counts, _ = np.histogram(at, bins=bins)
         self.at += counts
+
         lats = dataset["latitude"]
         counts, _ = np.histogram(lats, bins=self.lat_bins)
         self.lats += counts
 
+        scan_time = dataset["scan_time"].astype("datetime64[ns]")
+        lons = dataset["longitude"].data
+        local_time = (scan_time +
+                      (lons / 360 * 24 * 60 * 60).astype("timedelta64[s]"))
+        minutes = local_time.dt.hour * 60 + local_time.dt.minute.data
+        counts, _ = np.histogram(minutes, bins=self.time_bins)
+        self.local_time += counts
+
+        counts, _, _ = np.histogram2d(
+            lats.data.ravel(),
+            minutes.data.ravel(),
+            bins=(self.lat_bins, self.time_bins)
+        )
+        self.lat_local_time += counts
+
     def merge(self, other):
         """
         Merge the data of this statistic with that calculated in a different
@@ -1226,6 +1313,8 @@ class TrainingDataStatistics(Statistic):
             self.st = other.st
             self.at = other.at
             self.lats = other.lats
+            self.local_time = other.local_time
+            self.lat_local_time = other.lat_local_time
             self.sums_tcwv = other.sums_tcwv
             self.counts_tcwv = other.counts_tcwv
 
@@ -1241,6 +1330,8 @@ class TrainingDataStatistics(Statistic):
             self.st += other.st
             self.at += other.at
             self.lats += other.lats
+            self.local_time += other.local_time
+            self.lat_local_time += other.lat_local_time
 
     def save(self, destination):
         """
@@ -1317,6 +1408,8 @@ class TrainingDataStatistics(Statistic):
         data["surface_type"] = ("surface_type_bins",), self.st
         data["airmass_type"] = ("airmass_type_bins"), self.st
         data["latitudes"] = ("latitude_bins"), self.lats
+        data["local_time"] = ("time_bins"), self.local_time
+        data["lat_local_time"] = ("latitude_bins", "time_bins"), self.lat_local_time
 
         data = xr.Dataset(data)
 
@@ -1587,17 +1680,20 @@ class ObservationStatistics(Statistic):
     This statistic calculates TB distributions from L1C files.
     """
 
-    def __init__(self, conditional=None):
+    def __init__(self, conditional=None, statistics=None, gmi_range=True):
         """
         Args:
             conditional: If provided should identify a channel for which
                 conditional of all other channels will be calculated.
         """
+        self.statistics = statistics
+        self.gmi_range = gmi_range
         self.angle_bins = None
         self.has_angles = None
         self.tbs = None
         self.tb_bins = np.linspace(0, 400, 401)
-        self.lat_bins = np.linspace(-90, 90, 181)
+        self.lat_bins = LAT_BINS
+        self.time_bins = TIME_BINS
 
         self.t2m = np.zeros((18, 200), dtype=np.float32)
         self.t2m_bins = np.linspace(240, 330, 201)
@@ -1606,6 +1702,8 @@ class ObservationStatistics(Statistic):
         self.st = np.zeros(18, dtype=np.float32)
         self.at = np.zeros(4, dtype=np.float32)
         self.lats = np.zeros(180, dtype=np.float32)
+        self.local_time = np.zeros(24, dtype=np.float32)
+        self.lat_local_time = np.zeros((180, 24), dtype=np.float32)
 
     def _initialize_data(self, sensor, data):
         n_chans = sensor.n_chans
@@ -1651,12 +1749,21 @@ class ObservationStatistics(Statistic):
         """
         self.sensor = sensor
         dataset = open_file(filename)
+
+        if self.statistics is not None:
+            dataset = resample_scans(dataset, self.statistics)
+
         if self.tbs is None:
             self._initialize_data(sensor, dataset)
 
         st = dataset["surface_type"]
         for i in range(18):
+
             i_st = (st == i + 1).data
+            if self.gmi_range and self.statistics is None:
+                lats = dataset["latitude"].data
+                i_st *= (lats >= -65.5) * (lats <= 65.5)
+
             tcwv = dataset["total_column_water_vapor"].data[i_st]
 
             # Sensor with varying EIA (cross track).
@@ -1706,6 +1813,21 @@ class ObservationStatistics(Statistic):
         counts, _ = np.histogram(lats, bins=self.lat_bins)
         self.lats += counts
 
+        scan_time = dataset["scan_time"]
+        lons = dataset["longitude"]
+        local_time = (scan_time +
+                      (lons / 360 * 24 * 60 * 60).astype("timedelta64[s]"))
+        minutes = local_time.dt.hour.data * 60 + local_time.dt.minute.data
+        counts, _ = np.histogram(minutes, bins=self.time_bins)
+        self.local_time += counts
+
+        counts, _, _ = np.histogram2d(
+            lats.ravel(),
+            minutes.ravel(),
+            bins=(self.lat_bins, self.time_bins)
+        )
+        self.lat_local_time += counts
+
         dataset.close()
         del dataset
 
@@ -1722,6 +1844,8 @@ class ObservationStatistics(Statistic):
             self.st = other.st
             self.at = other.at
             self.lats = other.lats
+            self.local_time = other.local_time
+            self.lat_local_time = other.lat_local_time
         elif other.tbs is not None:
             self.tbs += other.tbs
             self.tbs_tcwv += other.tbs_tcwv
@@ -1730,6 +1854,8 @@ class ObservationStatistics(Statistic):
             self.st += other.st
             self.at += other.at
             self.lats += other.lats
+            self.local_time += other.local_time
+            self.lat_local_time += other.lat_local_time
 
     def save(self, destination):
         """
@@ -1790,6 +1916,8 @@ class ObservationStatistics(Statistic):
         data["surface_type"] = ("surface_type_bins",), self.st
         data["airmass_type"] = ("airmass_type_bins"), self.st
         data["latitudes"] = ("latitude_bins"), self.lats
+        data["local_time"] = ("time_bins"), self.local_time
+        data["lat_local_time"] = ("latitude_bins", "time_bins"), self.lat_local_time
 
         data = xr.Dataset(data)
 
@@ -1805,12 +1933,16 @@ class RetrievalStatistics(Statistic):
     This class calculates conditional distributions of retrieval results
     w. r. t. the two meter temperature and the total column water vapor.
     """
-    def __init__(self):
+    def __init__(self, statistics=None):
         """
         Args:
-            conditional: If provided should identify a channel for which
-                conditional of all other channels will be calculated.
+            statistics: If provide should be a 1 or 2-dimensional numpy array.
+                If the array is 1D, the weights in the array will be used to
+                resample the retrieval results by latitudes. If it is 2D,
+                the weight will be used to resample both latitude and local
+                time.
         """
+        self.statistics = statistics
         self.targets = {}
         self.sums_t2m = {}
         self.counts_t2m = {}
@@ -1859,8 +1991,10 @@ class RetrievalStatistics(Statistic):
             sensor: The sensor from which the data stems.
             filename: The path of the data to process.
         """
-        self.sensor = sensor
         dataset = open_file(filename)
+        if self.statistics is not None:
+            dataset = resample_scans(dataset, self.statistics)
+
         if len(self.targets) == 0:
             self._initialize_data(sensor, dataset)
 
@@ -2015,7 +2149,7 @@ class RetrievalStatistics(Statistic):
         data = xr.Dataset(data)
         destination = Path(destination)
         output_file = destination / (
-            f"retrieval_statistics_{self.sensor.name.lower()}" ".nc"
+            f"retrieval_statistics.nc"
         )
         data.to_netcdf(output_file)
 
@@ -2044,6 +2178,7 @@ def process_files(sensor, files, statistics, log_queue):
         for stat in statistics:
             try:
                 stat.process_file(sensor, file)
+                pass
             except Exception as exc:
                 LOGGER.error("Error during processing of %s: %s", file, exc)
 
